<?php

declare(strict_types=1);

namespace Drupal\menu_tree\Hook;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityFormInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Hook\Order\OrderAfter;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Template\Attribute;
use Drupal\menu_tree\MenuTreeItems;
use Drupal\menu_tree\NodeFormSubmitHandler;
use Drupal\node\NodeInterface;
use Drupal\node\NodeTypeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class Hooks.
 */
class Hooks implements ContainerInjectionInterface {

  use StringTranslationTrait;

  protected NodeFormSubmitHandler $nodeFormSubmitHandler;
  protected MenuTreeItems $menuTreeItems;
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * Hooks constructor.
   */
  public function __construct(
    NodeFormSubmitHandler $node_form_submit_handler,
    MenuTreeItems $menu_tree_items,
    ModuleHandlerInterface $module_handler
  ) {
    $this->nodeFormSubmitHandler = $node_form_submit_handler;
    $this->menuTreeItems = $menu_tree_items;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) : static {
    return new static(
      $container->get('menu_tree.node_form_submit_handler'),
      $container->get('menu_tree.items'),
      $container->get('module_handler')
    );
  }

  /**
   * Implements hook_help().
   */
  #[Hook('help')]
  function help($route_name, RouteMatchInterface $route_match) : string|\Stringable|array|null {
    if ($route_name == 'help.page.menu_tree') {
      $output = '';
      $output .= '<h3>' . t('About') . '</h3>';
      $output .= '<p>' . t('This modules replaces the standard widget for selecting Parent link on node add and edit forms with a tree widget.') . '</p>';
      $output .= '<h3>' . t('Supporting organizations:') . '</h3>';
      $output .= '<p>' . t('<a href="https://www.drupal.org//happiness">Happiness</a>') . '</p>';
      return $output;
    }
    return NULL;
  }

  /**
   * Implements hook_form_BASE_FORM_ID_alter().
   *
   * This function modifies the node form elements based on the configuration
   * of the 'menu_tree' module. It uses the settings for bundles and available
   * menus to determine the alterations needed for the menu-related form
   * elements. When applicable, it hides the default menu parent and weight
   * elements and adds a tree selector for menu placement.
   */
  #[Hook('form_node_form_alter', order: new OrderAfter(['menu_ui']))]
  function nodeFormAlter(&$form, FormStateInterface $form_state, $form_id) : void {
    /** @var NodeInterface $node */
    $node = $form_state->getFormObject()->getEntity();
    /** @var NodeTypeInterface $node_type */
    $node_type = $node->get('type')->entity;
    $use_tree_widget = $node_type->getThirdPartySetting('menu_tree', 'use_tree_widget', FALSE);

    // Is the tree widget enabled for this node type?
    if (!$use_tree_widget) {
      return;
    }

    // Get available menus from node.type.<bundle>.yml.
    $available_menus = $node_type->getThirdPartySetting('menu_ui', 'available_menus', []);

    // No menus configured for the bundle, do nothing.
    if (empty($available_menus)) {
      return;
    }

    // Assign the root element.
    $root_element = &$form['menu'];

    // If 'menu_ui_async_widget' exists and is configured for this content
    // type, add special logic to handle this.
    if ($this->moduleHandler->moduleExists('menu_ui_async_widget')) {
      $use_async_widget = $node_type->getThirdPartySetting('menu_ui_async_widget', 'use_async_widget', FALSE);
      if ($use_async_widget) {
        $root_element = &$form['menu']['container'];
      }
      if ($use_async_widget && !$form_state->get('menu_ui_async_widget')) {
        return;
      }
    }

    // Hide menu parent selector and weight elements.
    $root_element['link']['menu_parent']['#type'] = 'hidden';
    $root_element['link']['weight']['#type'] = 'hidden';

    $defaults = menu_ui_get_menu_link_defaults($node);
    $exclude = $defaults['menu_name'] . ':' . $defaults['id'];
    if (!$defaults['parent']) {
      $state = (object)[
        'class' => 'expand',
        'text' => $this->t('Expand all'),
      ];
    }
    else {
      $state = (object)[
        'class' => 'collapse',
        'text' => $this->t('Collapse all'),
      ];
    }

    // Get menus.
    $menus = [];
    foreach ($available_menus as $id) {
      $menus[] = $this->menuTreeItems->getLinks($id);
    }

    // Add the menu tree component.
    $attributes = new Attribute(['class' => 'menu-tree-wrapper']);
    $root_element['link']['menu_tree'] = [
      '#type' => 'component',
      // '#name' is required to fix class names in the theme wrapper 'form_element'.
      '#name' => 'menu-tree',
      '#component' => 'menu_tree:menu-tree',
      '#props' => [
        'menus' => $menus,
        'exclude' => $exclude,
        'selected' => $defaults['parent'],
        'attributes' => $attributes,
        'state' => $state,
      ],
      '#title' => $this->t('Parent link'),
      '#theme_wrappers' => ['form_element'],
      '#cache' => [
        'max-age' => 0,
      ],
    ];

    $root_element['link']['prev_sibling'] = [
      '#type' => 'hidden',
      '#value' => '',
    ];

    $root_element['link']['next_sibling'] = [
      '#type' => 'hidden',
      '#value' => '',
    ];

    // Add submit handler.
    foreach (array_keys($form['actions']) as $action) {
      if ($action != 'preview' && isset($form['actions'][$action]['#type']) && $form['actions'][$action]['#type'] === 'submit') {
        $form['actions'][$action]['#submit'][] = [$this->nodeFormSubmitHandler, 'handleFormSubmit'];
      }
    }
  }

  /**
   * Implements hook_form_FORM_ID_alter().
   *
   * Adds a checkbox to enable the tree widget on the node type settings form.
   *
   * @see \Drupal\node\Form\NodeTypeForm::form()
   */
  #[Hook('form_node_type_form_alter')]
  public function nodeTypeFormAlter(&$form, FormStateInterface $form_state, $form_id) : void {
    /** @var EntityFormInterface $form_object */
    $form_object = $form_state->getFormObject();
    /** @var NodeTypeInterface $type */
    $type = $form_object->getEntity();

    $form['menu']['use_tree_widget'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use tree widget for parent link'),
      '#default_value' => $type->getThirdPartySetting('menu_tree', 'use_tree_widget', FALSE),
      '#description' => $this->t('If checked, a tree widget will be used to select the parent link on node forms.'),
    ];

    $form['#entity_builders'][] = [$this, 'nodeTypeFormBuilder'];
  }

  /**
   * Entity builder for the node type form with menu options.
   */
  public function nodeTypeFormBuilder($entity_type, NodeTypeInterface $type, &$form, FormStateInterface $form_state) : void {
    $type->setThirdPartySetting('menu_tree', 'use_tree_widget', $form_state->getValue('use_tree_widget'));
  }

}
